---
title: LibreTime 3.0.0 alpha 3
---

import ReleaseHead from './\_release-head.md';

<ReleaseHead date="2017-10-18" version="3.0.0-alpha.3"/>

## :rocket: Features

- Debian 9 support in installer shell script
- Simplify configuration file structure
- Lots of work on Hungarian (hu_HU) translation
- Automatic Creation of Smartblocks and Playlists for new Podcasts
- Duplicate Smartblock functionality in context menu
- Better server preparation docs with firewall infos

## :bug: Bug fixes

- CORS responses work for all browsers and with TLS or on non default ports
- Fix update warning for pre-release candidates
- Dont install dev packages in installer for faster installs
- Some php 7.1 patches
- Better logging for analyzer and pypo
- Fix chunked podcast downloading
- Fix TLS timestamp updating

## :fire: Deprecation and removal

- Dropped support for Debian Wheezy

## :arrow_up: Upgrading

### Configuration files merging

The contents of `/etc/airtime/cloud_storage.conf` and `/etc/airtime/rabbitmq-analyzer.ini` have been moved to the main `/etc/airtime/airtime.ini` file and the env specific symlink is no longer needed. See below for instructions on how to migrate from a pre-release that created those files.

The configuration file structure has changed. Please move the contents of the `/etc/airtime/cloud_storage.conf` and `/etc/airtime/rabbitmq-analyzer.ini` files into the main `/etc/airtime/airtime.ini`. In all known cases you need to add the following sections to the file.

```ini
[current_backend]
storage_backend=file

[amazon_S3]
provider=amazon_S3
bucket=0
api_key=0
api_key_secret=0
```

You can then remove the files and the symlink.

```bash
rm -f \
  /etc/airtime/cloud_storage.conf \
  /etc/airtime/rabbitmq-analyzer.ini \
  /etc/airtime/production
```

Analyzer grabs all the needed info from the main `airtime.ini` file starting with `3.0.0-alpha.3`.

## :warning: Known issues

The following issues need a workaround for the time being. Please search the [issues](https://github.com/libretime/libretime/issues) before reporting problems not listed below.

### Outdated silan reports unreliable cue in/out information

Out of the box the installer installs a broken, outdated version of silan on most Debian based Platforms (ie. Ubuntu). This affects all distros supported by the installer except Debian unstable which has the silan 0.3.3 and CentOS which doesn't have upstream packages and you may either install from source or use the [0.3.3 packages](https://github.com/radiorabe/centos-rpm-silan) from [RaBe APEL](https://build.opensuse.org/project/show/home:radiorabe:audio).

You can check the version of silan by running with `silan --version` which should report `0.3.3` or with `sudo dpkg -s silan | awk '/Version/ {print $2}'` that will show the exact package version you installed. Please include this information if you file bugs concerning silan.

There are multiple workarounds to this issue and [a bug against the upstream](https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=855319) has been opened with Debian. Until this is sorted you should pre-install silan from a source you trust. If your distro hasn't updated to 0.3.3 yet you can also help by asking your distros maintainers for a bump.

Kyle Robbertze @paddatrapper has taken over maintainership of the silan packages in Debian as he is currently packaging LibreTime for Debian. Thank you very much Kyle.

The following solutions have been reported to work.

#### Silan from OBS build (https://github.com/libretime/libretime/issues/177#issuecomment-299195796)

Works for all relevant Debian and Ubuntu distros and installs silan `0.3.3~nmu1`.

```bash
sudo apt-get update
sudo apt-get install apt-transport-https

# install package signing key from obs
wget -qO- https://download.opensuse.org/repositories/home:/hairmare:/silan/Debian_7.0/Release.key \
  | apt-key add -

# add OBS repo to sources list (pick the distro you need)

# Debian Stretch
echo 'deb https://download.opensuse.org/repositories/home:/hairmare:/silan/Debian_9.0_standard/ ./' \
  > /etc/apt/sources.list.d/hairmare_silan.list
# Debian Jessie
echo 'deb https://download.opensuse.org/repositories/home:/hairmare:/silan/Debian_8.0 ./' \
  > /etc/apt/sources.list.d/hairmare_silan.list
# Ubuntu Xenial
echo 'deb https://download.opensuse.org/repositories/home:/hairmare:/silan/xUbuntu_16.04 ./' \
  > /etc/apt/sources.list.d/hairmare_silan.list
# Ubuntu Trusty
echo 'deb https://download.opensuse.org/repositories/home:/hairmare:/silan/xUbuntu_14.04 ./' \
  > /etc/apt/sources.list.d/hairmare_silan.list

# update local package database
apt-get update

# install silan 0.3.3 from obs packages
apt-get install silan
```

#### Local armhf builds for Raspberry Pi 3 (https://github.com/libretime/libretime/issues/214#issuecomment-305988355)

Since build.opensuse.org can't build Debian packages on arm due to missing dependencies, the `0.3.3~nmu1` arm package was built in a docker crossdev environment. This is reported to work on Debian Jessie on a Raspberry Pi 3 Model B.

```bash
curl -L -O https://github.com/libretime/libretime/files/1049738/silan_0.3.3.nmu1_armhf.deb.tar.gz
tar xvf silan_0.3.3.nmu1_armhf.deb.tar.gz
sudo dpkg -i silan_0.3.3~nmu1_armhf.deb
```

#### Legacy upstream silan packages (https://github.com/libretime/libretime/issues/197)

Legacy upstream hosts patched packages for Ubuntu Trusty on `apt.sourcefabric.org`. They install as `0.3.2~trusty~sfo-1`.

```bash
sudo apt-get update
sudo apt-get install apt-transport-https

sudo tee -a /etc/apt/sources.list <<EOD
deb https://apt.sourcefabric.org/ trusty main
EOD

sudo apt-get update
sudo apt-get install sourcefabric-keyring
sudo apt-get update
sudo apt-get install --reinstall silan=0.3.2~trusty~sfo-1
```

#### Remove silan completely (https://github.com/libretime/libretime/issues/193#issuecomment-299174997)

It's worth mentioning that you can disable cue point detection by removing silan from the system.

```bash
sudo apt-get uninstall silan
```

Reportedly this might wake the :cat2: (![happy-cuteness-overload-small](https://user-images.githubusercontent.com/116588/26853726-6d5bd6be-4b13-11e7-9dbd-9d7333e828c5.png) _aww_) or have other [side effects](https://github.com/libretime/libretime/issues/214#issuecomment-305748757).

### Liquidsoap >= 1.3.0 Support

LibreTime currently only supports liquidsoap `< 1.3.0` out of the box. If you install a current version of liquidsoap [using OPAM](https://www.liquidsoap.info/) or through the [Rabe Liquidsoap Distribution for CentOS (RaBe LSD)](https://build.opensuse.org/project/show/home:radiorabe:liquidsoap) you will most likely have liquidsoap `1.3.1` installed.

You can check your liquidsoap version by running `liquidsoap --version`.

If you already have liquidsoap >= 1.3.0 you have a couple of options.

#### Liquidsoap 1.3.0 Patchset (#192)

You can patch your installation of LibreTime to support liquidsoap 1.3.0.

An up to date patch is available through GitHub and can be applied to an unpacked tarball as follows.

```bash
cd libretime-3.0.0-alpha.3/
curl -L https://github.com/libretime/libretime/compare/main...radiorabe:feature/liquidsoap-1.3.0.patch | patch -p1
```

Git users can pull from the branch at https://github.com/libretime/libretime/compare/main...radiorabe:feature/liquidsoap-1.3.0 directly.

#### Install old liquidsoap from opam (#192)

You can downgrade an OPAM install of liquidsoap by running the following command.

```bash
opam install "liquidsoap<1.3.0"
```

### No watched folder support

Currently LibreTime doesn't support watching folders. Uploading files through the web interface works fine and can be automated via a REST API. Re-implementing watched folder support is on the roadmap. Please consider helping out with the code to help speed things along if you want to use the feature.

### No line in support

Line In recording can be enabled in the UI but doesn't currently work. This feature went missing from LibreTime due to the fact that we based our code off of the saas-dev branch of legacy upstream and support for recording hasn't been ported to the new airtime analyzer ingest system. #42 currently tracks the progress being made on line in recording.

### Playout won't work if locale is missing

Some minimal OS installs don't have a default locale configured. This only seems to affect some VPS installs as they often don't have a locale setup in the default images provided.

You can set up the locale using a combination of the following command. You might also want to consult the documentation of your VPS provider as it may contain an official way to set up locales when provisioning a VPS.

```bash
# Set locale using systemds localectl
localectl set-locale LANG="en_US.utf8"
```

these instructions don't seem to work on all Debian based distros so you might need to use `update-locale` as follows.

```bash
#Purge all locales but en_US.UTF-8
sudo locale-gen --purge en_US.UTF-8
#Populate LANGUAGE=
sudo update-locale LANGUAGE="en_US.UTF-8"
```

## :memo: Colophon

It's been awesome to see that the interest in LibreTime isn't dying off.

I recently did some cleaning up of the labels and milestones in the issue tracker and would like to ask everyone to help out with tagging issues and managing milestones.

Currently the milestones only contain issues likely to be finished in the development window of the respective version. This includes things that already have a pull-request or issues with a high priority due to their sensible or blocking nature. Feel free to request new labels and unversioned milestones if you feel you can use them.

That's all for these release notes. I'd also like to note that I plan on doing releases more often. They will be smaller but there will be less changes per release.
