---
title: LibreTime 3.0.0 alpha 5
---

import ReleaseHead from './\_release-head.md';

<ReleaseHead date="2018-10-12" version="3.0.0-alpha.5"/>

## :rocket: Features

- Interface improvements
- Raspbian 9 support in installer script
- Allow skipping RabbitMQ and PostgreSQL install in installer script with `--no-rabbitmq ` and `--no-postgres`

## :bug: Bug fixes

- Remove `[media-monitor]` section from default install
- Fix last help links that pointed to sourcefabric and update others to point to discourse
- Fix for ShoutCAST DNAS rejecting title updates when artist is missing
- Initial fixes for native IPv6 support
- Support modern pika versions
- Disable podcast subscription limit
- Return proper `Content-Length` for legacy database entries missing the info

## :fire: Deprecation and removal

- Removed Amazon S3 storage support

### Deprecation of Ubuntu 14.04 (Trusty)

This info is an early warning that we're deprecating a major os version.

Ubuntu 16.04 (Xenial) and Ubuntu 18.04 (Bionic) are now available and Trusty is EOL in April 2019.

## :arrow_up: Upgrading

### S3 storage support removal

The configuration file structure has changed. Remove the `amazon_s3` configuration section from the `/etc/airtime/airtime.conf` file, as S3 storage support has been removed.

## :warning: Known issues

The following issues may need a workaround for the time being. Please search the [issues](https://github.com/libretime/libretime/issues) before reporting problems not listed below.

### Media-Monitor config needs manual removing

If you are using the `install` script you should most likely remove the `[media-monitor]` config section from your `/etc/airtime/airtime.conf` file to ensure you don't run into the problems described in
https://github.com/libretime/libretime/issues/450. We recommend you do this before running the update since there are no known LibreTime releases that depend on the config value.

### Outdated silan reports unreliable cue in/out information

Out of the box the installer installs a broken, outdated version of silan on most Debian based Platforms (ie. Ubuntu). This affects all distros supported by the installer except Debian unstable which has the silan 0.3.3 and CentOS which doesn't have upstream packages and you may either install from source or use the [0.3.3 packages](https://github.com/radiorabe/centos-rpm-silan) from [RaBe APEL](https://build.opensuse.org/project/show/home:radiorabe:audio).

You can check the version of silan by running with `silan --version` which should report `0.3.3` or with `sudo dpkg -s silan | awk '/Version/ {print $2}'` that will show the exact package version you installed. Please include this information if you file bugs concerning silan.

There are multiple workarounds to this issue and [a bug against the upstream](https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=855319) has been fixed. Kyle Robbertze @paddatrapper has taken over maintainership of the silan packages in Debian as he is currently packaging LibreTime for Debian. The silan package is in testing as can be seen in the [Debian PTS](https://tracker.debian.org/pkg/silan). This section will get updated once the package is in stable.

Until this is sorted you should pre-install silan from a source you trust. If your distro hasn't updated to 0.3.3 yet you can also help by asking your distros maintainers for a bump.

The following solutions have been reported to work. If they don't work, please note that it's your responsibility to get working silan packages.

#### Silan from OBS build (https://github.com/libretime/libretime/issues/177#issuecomment-299195796)

Works for all relevant Debian and Ubuntu distros and installs silan `0.3.3~nmu1`.

```bash
sudo apt-get update
sudo apt-get install apt-transport-https

# install package signing key from obs
wget -qO- https://download.opensuse.org/repositories/home:/hairmare:/silan/Debian_7.0/Release.key \
  | apt-key add -

# add OBS repo to sources list (pick the distro you need)

# Debian Stretch
echo 'deb https://download.opensuse.org/repositories/home:/hairmare:/silan/Debian_9.0_standard/ ./' \
  > /etc/apt/sources.list.d/hairmare_silan.list
# Debian Jessie
echo 'deb https://download.opensuse.org/repositories/home:/hairmare:/silan/Debian_8.0 ./' \
  > /etc/apt/sources.list.d/hairmare_silan.list
# Ubuntu Xenial
echo 'deb https://download.opensuse.org/repositories/home:/hairmare:/silan/xUbuntu_16.04 ./' \
  > /etc/apt/sources.list.d/hairmare_silan.list
# Ubuntu Trusty (14.04) EOL in April 2019
echo 'deb https://download.opensuse.org/repositories/home:/hairmare:/silan/Ubuntu_14.10_standard/ ./' \
  > /etc/apt/sources.list.d/hairmare_silan.list

# update local package database
apt-get update

# install silan 0.3.3 from obs packages
apt-get install silan
```

#### Local armhf builds for Raspberry Pi 3 (https://github.com/libretime/libretime/issues/214#issuecomment-305988355)

Since build.opensuse.org can't build Debian packages on arm due to missing dependencies, the `0.3.3~nmu1` arm package was built in a docker crossdev environment. This is reported to work on Debian Jessie on a Raspberry Pi 3 Model B.

```bash
curl -L -O https://github.com/libretime/libretime/files/1049738/silan_0.3.3.nmu1_armhf.deb.tar.gz
tar xvf silan_0.3.3.nmu1_armhf.deb.tar.gz
sudo dpkg -i silan_0.3.3~nmu1_armhf.deb
```

#### Legacy upstream silan packages (https://github.com/libretime/libretime/issues/197)

Legacy upstream hosts patched packages for Ubuntu Trusty on `apt.sourcefabric.org`. They install as `0.3.2~trusty~sfo-1`.

```bash
sudo apt-get update
sudo apt-get install apt-transport-https

sudo tee -a /etc/apt/sources.list <<EOD
deb https://apt.sourcefabric.org/ trusty main
EOD

sudo apt-get update
sudo apt-get install sourcefabric-keyring
sudo apt-get update
sudo apt-get install --reinstall silan=0.3.2~trusty~sfo-1
```

#### Remove silan completely (https://github.com/libretime/libretime/issues/193#issuecomment-299174997)

It's worth mentioning that you can disable cue point detection by removing silan from the system.

```bash
sudo apt-get uninstall silan
```

Reportedly this might have [side effects](https://github.com/libretime/libretime/issues/214#issuecomment-305748757).

### Liquidsoap Support

LibreTime currently only supports liquidsoap `< 1.3.0` out of the box. If you install a current version of liquidsoap [using OPAM](https://www.liquidsoap.info/) or through the [Rabe Liquidsoap Distribution for CentOS (RaBe LSD)](https://build.opensuse.org/project/show/home:radiorabe:liquidsoap) you will most likely have liquidsoap `1.3.2` or `1.3.3` installed.

#352 reports that liquidsoap `< 1.3.0` can exhibit issues on some Debian based installs. One fix for the issue is to install liquidsoap `1.3.0` and to use the following patching steps. Kyle from silan fame is taking the needed steps to get liquidsoap maintained again in Debian so we expect the situation to improve over time.

You can check your liquidsoap version by running `liquidsoap --version`.

If you already have liquidsoap >= 1.3.0 you have a couple of options.

#### Liquidsoap 1.3.0 Patchset (#192)

You can patch your installation of LibreTime to support liquidsoap 1.3.0.

An up to date patch is available through GitHub and can be applied to an unpacked tarball as follows.

```bash
cd libretime-3.0.0-alpha.5/
curl -L https://github.com/libretime/libretime/compare/main...radiorabe:feature/liquidsoap-1.3.0.patch | patch -p1
```

Git users can pull from the branch at https://github.com/libretime/libretime/compare/main...radiorabe:feature/liquidsoap-1.3.0 directly.

#### Install old liquidsoap from opam (#192)

You can downgrade an OPAM install of liquidsoap by running the following command.

```bash
opam install "liquidsoap<1.3.0"
```

### No watched folder support

Currently LibreTime doesn't support watching folders. Uploading files through the web interface works fine and can be automated via a REST API. Re-implementing watched folder support is on the roadmap. Please consider helping out with the code to help speed things along if you want to use the feature.

### No line in support

Line In recording can be enabled in the UI but doesn't currently work. This feature went missing from LibreTime due to the fact that we based our code off of the saas-dev branch of legacy upstream and support for recording hasn't been ported to the new airtime analyzer ingest system. #42 currently tracks the progress being made on line in recording.

### Playout won't work if locale is missing

Some minimal OS installs don't have a default locale configured. This only seems to affect some VPS installs as they often don't have a locale setup in the default images provided.

You can set up the locale using a combination of the following command. You might also want to consult the documentation of your VPS provider as it may contain an official way to set up locales when provisioning a VPS.

```bash
# Set locale using systemds localectl
localectl set-locale LANG="en_US.utf8"
```

These instructions don't seem to work on all Debian based distros so you might need to use `update-locale` as follows.

```bash
#Purge all locales but en_US.UTF-8
sudo locale-gen --purge en_US.UTF-8
#Populate LANGUAGE=
sudo update-locale LANGUAGE="en_US.UTF-8"
```

### Lack of i18n toolchain is disturbing

Some translations might miss the tarball. They didn't get lost but the build chain needs fixing. Work is in #301 and additional work is needed as it has become clear that we proabably want to support bidirectional translation syncing with zanata.

## :memo: Colophon

Sorry for the _late_ release, RelEng has been busy with adulting and generally having a life as of lately. I'm still very much committed to LibreTime but can only find a limited amount of time to look into Issues due to being busy qualifying LibreTime for production workloads at my station. I plan on re-shifting my focus on pull request reviews and RelEng to address my lack of time going further.
